#include <stdarg.h>
#include <stdio.h>
#include <math.h>
#include <grx20.h>

typedef double vector[3];
typedef double vector2[2];

struct wall {
	vector2 start;
	vector2 end;
	vector2 n;
};

typedef struct wall wall;

enum {
	drawit = 0
};

void readin(char *, void *, char *, ...);
void parsecomment(void);
void loadfile(char *);

void *salloc(size_t);
void *normalize2(double [2]);

int wallcount;
wall *wa;

FILE *infile;
int sheight = 600;

int main(int argc, char *argv[]) {
	int i;
	int x1, y1, x2, y2;

	if (argc < 2) {
		fprintf(stderr, "No filename given\n");
		exit(2);
	}

	loadfile(argv[1]);

	if (drawit) {
		GrSetMode(GR_width_height_color_graphics, 800, sheight, 16L << 20);
		GrClearScreen(GrBlack());
		for (i=0; i<wallcount; i++) {
			x1 = (int) wa[i].start[0];
			y1 = (int) wa[i].start[1];
			x2 = (int) wa[i].end[0];
			y2 = (int) wa[i].end[1];
			GrLine(x1, y1, x2, y2, GrWhite());
			x1 = (x1 + x2) / 2;
			y1 = (y1 + y2) / 2;
			GrLine(x1, y1, x1 + wa[i].n[0] * 10, y1 + wa[i].n[1] * 10, GrWhite());
		}
		getchar();
	} else {
		printf("%d\n", wallcount);
		for (i=0; i<wallcount; i++) {
			printf("%lf %lf %lf %lf\n", wa[i].start[0], wa[i].start[1],
					wa[i].end[0], wa[i].end[1]);
			printf("%lf %lf\n", wa[i].n[0], wa[i].n[1]);
		}
	}
}

void loadfile(char *filename) {
	int i, j, k;

	if (!(infile = fopen(filename, "r"))) {
			perror("fopen");
			exit(1);
	}

	readin("%d", &wallcount, "number of walls");

	wa = (wall *) salloc(sizeof(wall) * wallcount);
	for (i=0; i<wallcount; i++) {
		readin("%lf", &wa[i].start[0], "wall #%d start x", i);
		readin("%lf", &wa[i].start[1], "wall #%d start y", i);
		readin("%lf", &wa[i].end[0], "wall #%d end x", i);
		readin("%lf", &wa[i].end[1], "wall #%d end y", i);
		wa[i].n[0] = wa[i].end[1] - wa[i].start[1];
		wa[i].n[1] = wa[i].start[0] - wa[i].end[0];
		normalize2(wa[i].n);
	}
}

void readin(char *s, void *f, char *format, ...) {
	va_list arg;
	char done;
	int check;
	char ch;

	done = 0;
	do {
		fscanf(infile, " %c", &ch);
		if (ch == '/') {
			parsecomment();
		} else {
			ungetc(ch, infile);
			check = fscanf(infile, s, f);
			if (!check) {
				va_start(arg, format);
				fprintf(stderr, "Error reading ");
				vfprintf(stderr, format, arg);
				va_end(arg);
				exit(1);
			}
			//printf("Read %s\n", thing);
			done++;
		}
	} while (!done);
}

void parsecomment(void) {
	char ch;
	char flag;

	if ((ch = getc(infile)) != '*') {
		fprintf(stderr, "Warning: bad comment\n");
	}
	flag = 0;
	do {
		do {
			ch = getc(infile);
		} while (ch != '*');
		if ((ch = getc(infile)) == '/') {
			flag++;
		}
	} while (!flag);
}

void *salloc(size_t size) {
	void *ptr;
	
	ptr = (void *) malloc(size);
	if (!ptr) {
		fprintf(stderr, "out of memory\n");
		exit(2);
	}

	return(ptr);
}

void *normalize2(double v[2]) {
	double n;

	n = sqrt(v[0] * v[0] + v[1] * v[1]);
	v[0] /= n;
	v[1] /= n;
}
